package org.freecompany.util.core;

import static java.lang.String.valueOf;

/**
 * A generically typed and immutable pair of elements.
 */
public class Pair< T, Y> {

	protected final T one;
	protected final Y two;

	public Pair( T one, Y two) {
		this.one = one;
		this.two = two;
	}

	public T one() { return one; }
	public Y two() { return two; }

	@Override
	public boolean equals( final Object object) {
		if ( object == null) return false;
		if ( this == object) return true;
		final Pair< ?, ?> other = ( Pair< ?, ?>) object;
		return ( one == other.one || one != null && one.equals( other.one)) && ( two == other.two || two != null && two.equals( other.two));
	}

	@Override
	public int hashCode() {
		return ( one == null ? 0 : one.hashCode()) ^ ( two == null ? 0 : two.hashCode());
	}

	@Override
	public String toString() {
		return new StringBuilder( "Pair[ ").append( valueOf( one)).append( ", ").append( valueOf( two)).append( "]").toString();
	}
}
