package org.freecompany.util.core;

import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;

public class RingBuffer< T> implements Iterable< T> {

	protected final T[] array;
	protected int size;
	protected int read;
	protected int write;

	@SuppressWarnings( "unchecked")
	public RingBuffer( final int capacity) {
		array = ( T[]) new Object[ capacity];
	}

	public synchronized void add( final T type) {
		array[ write] = type;
		if ( size < array.length) ++size;
		else if ( write == read) read = increment( read);
		write = increment( write);
	}

	public synchronized T remove() {
		if ( size > 0) {
			--size;
			try {
				return array[ read];
			} finally {
				array[ read] = null;
				read = increment( read);
			}
		}
		return null;
	}

	public int size() { return size; }

	@SuppressWarnings( "unchecked")
	public synchronized Iterator< T> iterator() {
		final T[] copy = ( T[]) new Object[ size];
		int index = read;
		for ( int x = 0; x < size; x++) {
			copy[ x] = array[ index];
			index = increment( index);
		}
		return Arrays.asList( copy).iterator();
	}

	private final int increment( final int index) {
		return ( index + 1) % array.length;
	}
}
